/*-----------------------------------------------------------------------------

 2002, Jeff M Soft und Hardware GmbH, All Rights Reserved

-----------------------------------------------------------------------------*/

#include "Module.h"
#include "SEMod_struct.h"
#include <assert.h>
#include "SEPin.h"
#include <stdlib.h>

// define some constants to make referencing in/outs clearer

#define PN_SHAPE 0
#define	PN_GUICOM 1
#define PN_INDEX 2
#define PN_HIGHV 3
#define PN_LOWV 4
#define PN_OUT 5
#define NUMSLIDERS 16

//#define DEFAULT_VALUE "(0.0000)(0.0000)(0.0000)(0.0000)(0.0000)(0.0000)(0.0000)(0.0000)(0.0000)(0.0000)(0.0000)(0.0000)(0.0000)(0.0000)(0.0000)(0.0000)";


void Module::open()
{
	SEModule_base::open();

	// choose which function is used to process audio
	SET_PROCESS_FUNC(Module::sub_process);

	modpos_array = new float [ NUMSLIDERS * sizeof(float) ];

	last_count = 20;

	getPin(PN_OUT)->TransmitStatusChange( SampleClock(), ST_STATIC);
}


Module::~Module()
{
	delete modpos_array;
}

Module::Module(seaudioMasterCallback2 seaudioMaster, void *p_resvd1) : SEModule_base(seaudioMaster, p_resvd1)
{
}

void Module::OnLookupTableChanged()
{

	ChooseProcess();

	if( getPin(PN_INDEX)->getStatus() != ST_RUN )
	{
		getPin(PN_OUT)->TransmitStatusChange( SampleClock(), ST_STATIC );
	}
	

}

// The most important part, processing the audio
void Module::sub_process(long buffer_offset, long sampleFrames )
{
	// assign some pointers to your in/output buffers. usually blocks (array) of 96 samples
    float *in1  = buffer_offset + input1_buffer;
	float *in2  = buffer_offset + input2_buffer;
	float *in3  = buffer_offset + input3_buffer;
    float *out1 = buffer_offset + output1_buffer;

	while(--sampleFrames >= 0) 
    {

		float c = *in1 * 10.f + 0.1f;
		int count = c - 1;
		float x = 1.f - modpos_array[count];
		*out1 = (x *(*in2 - *in3)/1.f);
		
//		if (count != last_count) send_index_to_gui(count);

		if (count != last_count)
		{
			// inform SE pin's value has changed. SE will send the new value to the GUI
			index_to_gui = (int *) count ;
			getPin(PN_GUICOM)->TransmitStatusChange( SampleClock(), ST_STATIC );
		}

		last_count = count;
		in1++;
		in2++;
		in3++;
		out1++;
    }
}

void Module::sub_process_static(long buffer_offset, long sampleFrames )
{
	sub_process(buffer_offset, sampleFrames);
	static_count = static_count - sampleFrames;
	if( static_count <= 0 )
	{
		CallHost(seaudioMasterSleepMode);
	}
}

// convert string value into segment list
void Module::UpdatePositions( float *modpos_array, const SeSdkString &p_values)
{	

	SeSdkString s(p_values);

	// convert CString of numbers to array of screen co-ords
	int i = 0;
	while( s.length() > 0 && i < NUMSLIDERS )
	{
		int p1 = s.find("(");
		int p2 = s.find(")");
		if( p2 < 1 )
			return;

		float slider_val = StringToFloat( s.substr(p1+1,p2-1) );// Check this!
		modpos_array[i] = slider_val;

		i++;

		p2++;
		s = s.substr(p2, s.length() - p2);
	}
}


// describe the pins (plugs and parameters)
bool Module::getPinProperties (long index, SEPinProperties* properties)
{
	switch( index )								// !!TODO!! list your in / out plugs
	{
	case 0:
		properties->name				= "shape";
		properties->variable_address	= &shape_ptr;
		properties->direction			= DR_IN;
		properties->datatype			= DT_TEXT;
//		properties->default_value		= "";
		properties->flags				= IO_UI_COMMUNICATION_DUAL;//| IO_PRIVATE;
		break;
	case 1: // hidden pin that sends data from DSP to the GUI module
        properties->name				= "gui_com_pin";
        properties->variable_address	= &index_to_gui;
		properties->direction			= DR_OUT;
        properties->datatype			= DT_INT;
        properties->flags				= IO_PATCH_STORE|IO_HIDE_PIN;
        break;
	// GUI PIN. Appears only on GUI object
    case 2: // this GUI pin receives data from the DSP Module
		properties->name				= "Val from DSP";
        properties->direction			= DR_IN;
        properties->datatype			= DT_INT;
        properties->flags				= IO_PATCH_STORE|IO_UI_COMMUNICATION|IO_HIDE_PIN;
        break;
	case 3:
		properties->name				= "Index";
		properties->variable_address	= &input1_buffer;
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
		break;
	case 4:
		properties->name				= "High Value";
		properties->variable_address	= &input2_buffer;
		properties->default_value		= "10.0";
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
		break;
	case 5:
		properties->name				= "Low Value";
		properties->variable_address	= &input3_buffer;
		properties->default_value		= "0";
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
		break;
	case 6:
		properties->name				= "Output";
		properties->variable_address	= &output1_buffer;
		properties->direction			= DR_OUT;
		properties->datatype			= DT_FSAMPLE;
		break;


	default:
		return false; // host will ask for plugs 0,1,2,3 etc. return false to signal when done
	};

	return true;
}



void Module::OnPlugStateChange(SEPin *pin)
{
	
	if( pin->getPinID() == PN_SHAPE )
	{
		UpdatePositions(modpos_array, shape_ptr);
		OnLookupTableChanged();
	}

	if( pin->getPinID() == PN_INDEX )
	{
		ChooseProcess();
		getPin(PN_OUT)->TransmitStatusChange( SampleClock(), getPin(PN_INDEX)->getStatus() );
	}
}

void Module::ChooseProcess()
{
	if( getPin(PN_INDEX)->getStatus() == ST_RUN )
	{
		SET_PROCESS_FUNC(Module::sub_process);
	}
	else
	{
		static_count = getBlockSize();
		SET_PROCESS_FUNC(Module::sub_process_static);
	}
}

float Module::StringToFloat(const SeSdkString &p_string)
{
	char * temp;	// convert string to float
	return (float) strtod( p_string.data(), &temp );
}

/*
int Module::Handle(void)
{
	return CallHost(seaudioMasterGetModuleHandle);
}



void Module::SendStringToGui( int p_msg_id, int p_length, unsigned char *p_data )
{
	CallHost(seaudioMasterSendStringToGui, p_length, p_msg_id, p_data );
}

void Module::send_index_to_gui(int what)
{
	char buffer [33];
	itoa (what,buffer,10);
	SendStringToGui( id_to_int('o','m','o','d'),sizeof(buffer) ,  (unsigned char *)&buffer  );
}
*/
/*
int Module::truncate(float flt)
{
  int i;
  static const double half = 0.5f;
  _asm
  {
     fld flt
     fsub half
     fistp i
  }
  return i;
}
*/